#version 330 core

layout(location = 0) out vec4 FragColor;

in vec2 TexCoords;

uniform sampler2D gPosition;
uniform sampler2D gNormal;
uniform sampler2D gAlbedoSpec;

// Light properties
struct PointLight {
    vec3 Position; // View-space position
    vec3 Color;    // Light color
    float ambientIntensity;
    float diffuseIntensity;
};


struct SpotLight {
    vec3 Position; // View-space position
    vec3 Color;    // Light color
    float ambientIntensity;
    float diffuseIntensity;
    vec3 Direction;
    float CutOff;   // Cosine of inner cutoff angle
    float OuterCutOff; // Cosine of outer cutoff angle
};

uniform PointLight pointLights[1]; // Assume a maximum of 32 lights
uniform int numPointLights;
uniform SpotLight spotLights[14];
uniform int numSpotLights;


struct DirectionalLight {
    vec3 direction;        // Already in view space
    vec3 color;            // Light color
    float ambientIntensity;
    float diffuseIntensity;
};

uniform DirectionalLight directionalLight;
uniform vec3 viewPos; // Camera position in view space

void main() {
    // Retrieve view-space fragment position and normal from textures
    vec3 FragPosView = texture(gPosition, TexCoords).rgb; // View-space position
    vec3 NormalView = normalize(texture(gNormal, TexCoords).rgb); // View-space normal
    float Specular = texture(gAlbedoSpec, TexCoords).a; // Specular component
    vec3 lighting = vec3(0.0); // Initialize lighting to zero
    vec3 viewDir = normalize(-FragPosView); // View direction is negative Z in view space

    // Directional light calculation
    vec3 lightDir = normalize(-directionalLight.direction); // Light direction in view space
    float diff = max(dot(NormalView, lightDir), 0.0);
    vec3 diffuse = diff * directionalLight.color * directionalLight.diffuseIntensity;
    // Specular
    vec3 reflectDir = reflect(-lightDir, NormalView);
    float spec = pow(max(dot(viewDir, reflectDir), 0.0), 16.0);
    vec3 specular = directionalLight.color * spec * Specular * directionalLight.diffuseIntensity;
    // Ambient
    vec3 ambient = directionalLight.color * directionalLight.ambientIntensity;
    lighting += (ambient + diffuse + specular);

    // Point lights calculation
    for (int i = 0; i < numPointLights; ++i) {
        // Light position in view space
        vec3 lightPosView = pointLights[i].Position;

        // Calculate light direction in view space
        lightDir = normalize(lightPosView - FragPosView);
        diff = max(dot(NormalView, lightDir), 0.0);
        diffuse = diff * pointLights[i].Color * pointLights[i].diffuseIntensity;

        // Specular component
        reflectDir = reflect(-lightDir, NormalView);
        spec = pow(max(dot(viewDir, reflectDir), 0.0), 16.0);
        specular = pointLights[i].Color * spec * Specular * pointLights[i].diffuseIntensity;

        // Ambient component
        ambient = pointLights[i].Color * pointLights[i].ambientIntensity;

        // Attenuation calculation
        float distance = length(lightPosView - FragPosView);
        float attenuation = 1.0 / (distance * distance);

        diffuse *= attenuation;
        specular *= attenuation;
        ambient *= attenuation; // Apply attenuation to ambient

        lighting += ambient + diffuse + specular;
    }


    for (int i = 0; i < numSpotLights; ++i) {
        vec3 lightPosView = spotLights[i].Position;
        vec3 spotDir = normalize(spotLights[i].Direction);
        lightDir = normalize(lightPosView - FragPosView);

        // Spotlight cutoff
        float theta = dot(lightDir, -spotDir);
        float epsilon = spotLights[i].CutOff - spotLights[i].OuterCutOff;
        float intensity = clamp((theta - spotLights[i].OuterCutOff) / epsilon, 0.0, 1.0);

        if (theta > spotLights[i].OuterCutOff) {
            float diff = max(dot(NormalView, lightDir), 0.0);
            vec3 diffuse = diff * spotLights[i].Color * spotLights[i].diffuseIntensity;

            // Specular
            vec3 reflectDir = reflect(-lightDir, NormalView);
            float spec = pow(max(dot(viewDir, reflectDir), 0.0), 16.0);
            vec3 specular = spotLights[i].Color * spec * Specular * spotLights[i].diffuseIntensity;

            // Ambient
            vec3 ambient = spotLights[i].Color * spotLights[i].ambientIntensity;

            // Attenuation
            float distance = length(lightPosView - FragPosView);
            float attenuation = 1.0 / (distance * distance);

            diffuse *= attenuation * intensity;
            specular *= attenuation * intensity;
            ambient *= attenuation * intensity;

            lighting += ambient + diffuse + specular;
        }
    }


    FragColor = vec4(lighting, 1.0);
}
